(*
 * SNU 4190.310 Programming Languages (Fall 2006)
 *
 * Main Interface for M
 *)

val print_m_code = ref false
val check_type = ref false
val spec = [
    ("-pp", Arg.Set print_m_code, "Print M program"),
    ("-ty", Arg.Set check_type, "Check types before running (with low fat M)")
]
val usage = "Usage: run [<options>] <M file> \n<options> are: "

fun run () =
    let val src = ref ""
        val _ = Arg.parse spec
                (fn x => if Sys.file_exists x then src := x
                    else raise Arg.Bad (x^": No such file")) usage
    in  
        Error.init ();
        let val lexbuf =
            Lexing.from_channel (if !src = "" then stdin else open_in !src)
            val pgm = Parser.program Lexer.start lexbuf
        in  if !print_m_code then (
                print_string "== Input Program ==\n"; 
                M_Printer.print pgm;
                print_newline()
            );
            if !check_type then (
                print_string "== Type Checking ==\n";
                M_Printer.printTypes (M_Checker.check pgm);
                print_string "== Running with Low Fat M ==\n";
                M_LowFat.run pgm
            ) else (
                print_string "== Running with Vanilla M ==\n";
                M_Vanilla.run pgm
            )
        end handle v => Error.handle_exn v
    end

val _ = Printexc.catch run () 
